Crate hugr_passes

Crate hugr_passes 

Compilation passes acting on the HUGR program representation.

This crate is deprecated. Use [`tket::passes`](https://docs.rs/tket/latest/tket/passes/index.html) instead.

Re-exports

pub use composable::ComposablePass;Deprecated
pub use composable::InScope;Deprecated
pub use composable::PassScope;Deprecated
pub use dead_code::DeadCodeElimPass;Deprecated
pub use force_order::force_order;Deprecated
pub use force_order::force_order_by_key;Deprecated
pub use inline_funcs::inline_acyclic;Deprecated
pub use lower::lower_ops;Deprecated
pub use lower::replace_many_ops;Deprecated
pub use non_local::ensure_no_nonlocal_edges;Deprecated
pub use replace_types::ReplaceTypes;Deprecated
pub use untuple::UntuplePass;Deprecated

Modules

composableDeprecated
Compiler passes and utilities for composing them.
const_foldDeprecated
Constant-folding pass. An (example) use of the dataflow analysis framework.
dataflowDeprecated
Dataflow analysis of Hugrs.
dead_codeDeprecated
Pass for removing dead code, i.e. that computes values that are then discarded
force_orderDeprecated
Provides force_order, a tool for fixing the order of nodes in a Hugr.
hashDeprecated
Hugr hashing.
inline_dfgsDeprecated
Provides InlineDFGsPass, a pass for inlining all DFGs in a Hugr.
inline_funcsDeprecated
Contains a pass to inline calls to selected functions in a Hugr.
lowerDeprecated
Passes to lower operations in a HUGR.
nest_cfgsDeprecated
Nest CFGs
non_localDeprecated
This module provides functions for finding non-local edges in a Hugr and converting them to local edges.
normalize_cfgsDeprecated
CFG normalizations.
redundant_order_edgesDeprecated
A pass for removing redundant order edges in a Hugr region.
replace_typesDeprecated
Replace types with other types across the Hugr. See ReplaceTypes and Linearizer.
untupleDeprecated
Pass for removing redundant tuple pack->unpack operations.

Structs

MonomorphizePassDeprecated
Replaces calls to polymorphic functions with calls to new monomorphic instantiations of the polymorphic ones.
RemoveDeadFuncsPassDeprecated
A configuration for the Dead Function Removal pass.

Enums

RemoveDeadFuncsErrorDeprecated
Errors produced by RemoveDeadFuncsPass.

Functions

mangle_nameDeprecated
Produce a mangled name for a monomorphized instance of a function.