DeadCodeElimPass

Struct DeadCodeElimPass 

pub struct DeadCodeElimPass<H: HugrView> { /* private fields */ }
๐Ÿ‘ŽDeprecated since 0.26.2:

hugr-passes is deprecated. Use tket::passes instead

Configuration for Dead Code Elimination pass

Implementations

Source

impl<H: HugrView> DeadCodeElimPass<H>

pub fn set_preserve_callback(self, cb: Arc<PreserveCallback<H>>) -> Self

๐Ÿ‘ŽDeprecated since 0.26.2:

hugr-passes is deprecated. Use tket::passes instead

Allows setting a callback that determines whether a node must be preserved (even when its result is not used)

pub fn with_entry_points( self, entry_points: impl IntoIterator<Item = H::Node>, ) -> Self

๐Ÿ‘ŽDeprecated since 0.26.2:

hugr-passes is deprecated. Use tket::passes instead

Mark some nodes as starting points for analysis, i.e. so we cannot eliminate any code used to evaluate these nodes. (E.g. nodes at which we may start executing the Hugr.)

Other starting points are added according to the PassScope.