Compilation passes acting on the HUGR program representation.
This crate is deprecated. Use [`tket::passes`](https://docs.rs/tket/latest/tket/passes/index.html) instead.
Re-exports
pub use composable::ComposablePass;Deprecated pub use composable::InScope;Deprecated pub use composable::PassScope;Deprecated pub use dead_code::DeadCodeElimPass;Deprecated pub use force_order::force_order;Deprecated pub use force_order::force_order_by_key;Deprecated pub use inline_funcs::inline_acyclic;Deprecated pub use lower::lower_ops;Deprecated pub use lower::replace_many_ops;Deprecated pub use non_local::ensure_no_nonlocal_edges;Deprecated pub use replace_types::ReplaceTypes;Deprecated pub use untuple::UntuplePass;Deprecated
Modules
- composable
Deprecated - Compiler passes and utilities for composing them.
- const_
fold Deprecated - Constant-folding pass. An (example) use of the dataflow analysis framework.
- dataflow
Deprecated - Dataflow analysis of Hugrs.
- dead_
code Deprecated - Pass for removing dead code, i.e. that computes values that are then discarded
- force_
order Deprecated - Provides
force_order, a tool for fixing the order of nodes in a Hugr. - hash
Deprecated - Hugr hashing.
- inline_
dfgs Deprecated - Provides
InlineDFGsPass, a pass for inlining all DFGs in a Hugr. - inline_
funcs Deprecated - Contains a pass to inline calls to selected functions in a Hugr.
- lower
Deprecated - Passes to lower operations in a HUGR.
- nest_
cfgs Deprecated - Nest CFGs
- non_
local Deprecated - This module provides functions for finding non-local edges in a Hugr and converting them to local edges.
- normalize_
cfgs Deprecated - CFG normalizations.
- redundant_
order_ edges Deprecated - A pass for removing redundant order edges in a Hugr region.
- replace_
types Deprecated - Replace types with other types across the Hugr. See
ReplaceTypesand Linearizer. - untuple
Deprecated - Pass for removing redundant tuple pack->unpack operations.
Structs
- Monomorphize
Pass Deprecated - Replaces calls to polymorphic functions with calls to new monomorphic instantiations of the polymorphic ones.
- Remove
Dead Funcs Pass Deprecated - A configuration for the Dead Function Removal pass.
Enums
- Remove
Dead Funcs Error Deprecated - Errors produced by
RemoveDeadFuncsPass.
Functions
- mangle_
name Deprecated - Produce a mangled name for a monomorphized instance of a function.