#[non_exhaustive]pub enum NodeTemplate {
SingleOp(OpType),
CompoundOp(Box<Hugr>),
LinkedHugr(Box<Hugr>, NameLinkingPolicy),
Call(Node, Vec<TypeArg>),
}hugr-passes is deprecated. Use tket::passes instead
A recipe for creating a dataflow Node - as a new child of a DataflowParent
or in order to replace an existing node.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
SingleOp(OpType)
hugr-passes is deprecated. Use tket::passes instead
A single node - so if replacing an existing node, change only the op.
An error will be raised if the new op has a static inport; use Self::LinkedHugr instead.
CompoundOp(Box<Hugr>)
hugr-passes is deprecated. Use tket::passes instead
Defines a sub-Hugr whose entrypoint-subtree to insert, the entrypoint (which must be
a CFG, Conditional, DFG or TailLoop) becoming (/replacing) the desired Node.
Note 1. CompoundOp will be of limited use for polymorphic ops (before monomorphization)
because the new subtree will not be able to use type variables present in the
parent Hugr or previous op.
Edges incoming to the entrypoint subtree will be disconnected.
It is recommended to use Self::LinkedHugr instead.
LinkedHugr(Box<Hugr>, NameLinkingPolicy)
hugr-passes is deprecated. Use tket::passes instead
Defines a sub-Hugr to insert, whose entrypoint becomes (or replaces) the desired Node. Other children of the Hugr reachable from the entrypoint will also be inserted according to the specified linking policy.
Call(Node, Vec<TypeArg>)
Use LinkedHugr with Call entrypoint and FuncDecl
A Call to an existing function.
Implementations
Sourceimpl NodeTemplate
impl NodeTemplate
Sourcepub fn linked_hugr(h: impl Into<Hugr>) -> Self
👎Deprecated since 0.26.2: hugr-passes is deprecated. Use tket::passes instead
pub fn linked_hugr(h: impl Into<Hugr>) -> Self
hugr-passes is deprecated. Use tket::passes instead
Creates a Self::LinkedHugr from the given Hugr with the default linking policy.
Sourcepub fn call_to_function(
func_def: Hugr,
type_args: &[TypeArg],
) -> Result<NodeTemplate, BuildError>
👎Deprecated since 0.26.2: hugr-passes is deprecated. Use tket::passes instead
pub fn call_to_function( func_def: Hugr, type_args: &[TypeArg], ) -> Result<NodeTemplate, BuildError>
hugr-passes is deprecated. Use tket::passes instead
Creates a NodeTemplate::LinkedHugr creating a call node to the given
function definition or declaration.
Returns a BuildError::UnexpectedType if the given Hugr does not have
a function definition or declaration as entrypoint.
Sourcepub fn add_hugr(
self,
hugr: &mut impl HugrMut<Node = Node>,
parent: Node,
) -> Result<Node, BuildError>
👎Deprecated since 0.26.2: hugr-passes is deprecated. Use tket::passes instead
pub fn add_hugr( self, hugr: &mut impl HugrMut<Node = Node>, parent: Node, ) -> Result<Node, BuildError>
hugr-passes is deprecated. Use tket::passes instead
Adds this instance to the specified HugrMut as a new node or subtree under a
given parent, returning the unique new child (of that parent) thus created
Panics
- If
parentis not in thehugr
Errors
- If
selfis aSelf::Calland the target Node either- is neither a
FuncDefnnor aFuncDecl - has a
signaturewhich the type-args of theSelf::Calldo not match
- is neither a
Sourcepub fn add(
self,
dfb: &mut impl Dataflow,
inputs: impl IntoIterator<Item = Wire>,
) -> Result<BuildHandle<DataflowOpID>, BuildError>
👎Deprecated since 0.26.2: hugr-passes is deprecated. Use tket::passes instead
pub fn add( self, dfb: &mut impl Dataflow, inputs: impl IntoIterator<Item = Wire>, ) -> Result<BuildHandle<DataflowOpID>, BuildError>
hugr-passes is deprecated. Use tket::passes instead
Adds this instance to the specified Dataflow builder as a new node or subtree